@@ -145,17 +145,6 @@ LogicalClockShmemSize(void)
 void
 InitializeClusterClockMem(void)
 {
-	/* On PG 15 and above, we use shmem_request_hook_type */
-	#if PG_VERSION_NUM < PG_VERSION_15
-
-	/* allocate shared memory for pre PG-15 versions */
-	if (!IsUnderPostmaster)
-	{
-		RequestAddinShmemSpace(LogicalClockShmemSize());
-	}
-
-	#endif
-
 	prev_shmem_startup_hook = shmem_startup_hook;
 	shmem_startup_hook = LogicalClockShmemInit;
 }
@@ -328,7 +317,7 @@ GetHighestClockInTransaction(List *nodeConnectionList)
 {
 	MultiConnection *connection = NULL;
 
-	foreach_ptr(connection, nodeConnectionList)
+	foreach_declared_ptr(connection, nodeConnectionList)
 	{
 		int querySent =
 			SendRemoteCommand(connection, "SELECT citus_get_node_clock();");
@@ -349,7 +338,7 @@ GetHighestClockInTransaction(List *nodeConnectionList)
 							globalClockValue->counter)));
 
 	/* fetch the results and pick the highest clock value of all the nodes */
-	foreach_ptr(connection, nodeConnectionList)
+	foreach_declared_ptr(connection, nodeConnectionList)
 	{
 		bool raiseInterrupts = true;
 
