@@ -185,8 +185,6 @@ PreprocessAlterDatabaseStmt(Node *node, const char *queryString,
 }
 
 
-#if PG_VERSION_NUM >= PG_VERSION_15
-
 /*
  * PreprocessAlterDatabaseSetStmt is executed before the statement is applied to the local
  * postgres instance.
@@ -217,9 +215,6 @@ PreprocessAlterDatabaseRefreshCollStmt(Node *node, const char *queryString,
 }
 
 
-#endif
-
-
 /*
  * GetDatabaseAddressFromDatabaseName gets the database name and returns the ObjectAddress
  * of the database.
