@@ -294,6 +294,17 @@ citus_move_shard_placement(PG_FUNCTION_ARGS)
 	CheckCitusVersion(ERROR);
 	EnsureCoordinator();
 
+	List *referenceTableIdList = NIL;
+
+	if (HasNodesWithMissingReferenceTables(&referenceTableIdList))
+	{
+		ereport(ERROR, (errmsg("there are missing reference tables on some nodes"),
+						errhint("Copy reference tables first with "
+								"replicate_reference_tables() or use "
+								"citus_rebalance_start() that will do it automatically."
+								)));
+	}
+
 	int64 shardId = PG_GETARG_INT64(0);
 	char *sourceNodeName = text_to_cstring(PG_GETARG_TEXT_P(1));
 	int32 sourceNodePort = PG_GETARG_INT32(2);
@@ -492,7 +503,7 @@ TransferShards(int64 shardId, char *sourceNodeName,
 	DropOrphanedResourcesInSeparateTransaction();
 
 	ShardInterval *colocatedShard = NULL;
-	foreach_ptr(colocatedShard, colocatedShardList)
+	foreach_declared_ptr(colocatedShard, colocatedShardList)
 	{
 		/*
 		 * This is to prevent any race condition possibility among the shard moves.
@@ -519,7 +530,7 @@ TransferShards(int64 shardId, char *sourceNodeName,
 	 * metadata workers.
 	 */
 	colocatedShard = NULL;
-	foreach_ptr(colocatedShard, colocatedShardList)
+	foreach_declared_ptr(colocatedShard, colocatedShardList)
 	{
 		uint64 colocatedShardId = colocatedShard->shardId;
 		uint32 groupId = GroupForNode(targetNodeName, targetNodePort);
@@ -612,7 +623,7 @@ InsertCleanupRecordsForShardPlacementsOnNode(List *shardIntervalList,
 											 int32 groupId)
 {
 	ShardInterval *shardInterval = NULL;
-	foreach_ptr(shardInterval, shardIntervalList)
+	foreach_declared_ptr(shardInterval, shardIntervalList)
 	{
 		/* get shard name */
 		char *qualifiedShardName = ConstructQualifiedShardName(shardInterval);
@@ -649,7 +660,7 @@ IsShardListOnNode(List *colocatedShardList, char *targetNodeName, uint32 targetN
 	 * We exhaustively search all co-located shards
 	 */
 	ShardInterval *shardInterval = NULL;
-	foreach_ptr(shardInterval, colocatedShardList)
+	foreach_declared_ptr(shardInterval, colocatedShardList)
 	{
 		uint64 shardId = shardInterval->shardId;
 		List *placementList = ActiveShardPlacementListOnGroup(shardId,
@@ -672,7 +683,7 @@ static void
 LockColocatedRelationsForMove(List *colocatedTableList)
 {
 	Oid colocatedTableId = InvalidOid;
-	foreach_oid(colocatedTableId, colocatedTableList)
+	foreach_declared_oid(colocatedTableId, colocatedTableList)
 	{
 		LockRelationOid(colocatedTableId, ShareUpdateExclusiveLock);
 	}
@@ -688,7 +699,7 @@ ErrorIfForeignTableForShardTransfer(List *colocatedTableList,
 									ShardTransferType transferType)
 {
 	Oid colocatedTableId = InvalidOid;
-	foreach_oid(colocatedTableId, colocatedTableList)
+	foreach_declared_oid(colocatedTableId, colocatedTableList)
 	{
 		if (IsForeignTable(colocatedTableId))
 		{
@@ -714,7 +725,7 @@ EnsureAllShardsCanBeCopied(List *colocatedShardList,
 						   char *targetNodeName, uint32 targetNodePort)
 {
 	ShardInterval *colocatedShard = NULL;
-	foreach_ptr(colocatedShard, colocatedShardList)
+	foreach_declared_ptr(colocatedShard, colocatedShardList)
 	{
 		uint64 colocatedShardId = colocatedShard->shardId;
 
@@ -1109,7 +1120,7 @@ void
 BlockWritesToShardList(List *shardList)
 {
 	ShardInterval *shard = NULL;
-	foreach_ptr(shard, shardList)
+	foreach_declared_ptr(shard, shardList)
 	{
 		/*
 		 * We need to lock the referenced reference table metadata to avoid
@@ -1280,7 +1291,7 @@ static void
 EnsureTableListOwner(List *tableIdList)
 {
 	Oid tableId = InvalidOid;
-	foreach_oid(tableId, tableIdList)
+	foreach_declared_oid(tableId, tableIdList)
 	{
 		EnsureTableOwner(tableId);
 	}
@@ -1295,7 +1306,7 @@ static void
 ErrorIfReplicatingDistributedTableWithFKeys(List *tableIdList)
 {
 	Oid tableId = InvalidOid;
-	foreach_oid(tableId, tableIdList)
+	foreach_declared_oid(tableId, tableIdList)
 	{
 		List *foreignConstraintCommandList =
 			GetReferencingForeignConstaintCommands(tableId);
@@ -1366,7 +1377,7 @@ CopyShardTablesViaLogicalReplication(List *shardIntervalList, char *sourceNodeNa
 	 * target node. We do not create the indexes yet.
 	 */
 	ShardInterval *shardInterval = NULL;
-	foreach_ptr(shardInterval, shardIntervalList)
+	foreach_declared_ptr(shardInterval, shardIntervalList)
 	{
 		Oid relationId = shardInterval->relationId;
 		uint64 shardId = shardInterval->shardId;
@@ -1433,7 +1444,7 @@ CopyShardTablesViaBlockWrites(List *shardIntervalList, char *sourceNodeName,
 
 	/* iterate through the colocated shards and copy each */
 	ShardInterval *shardInterval = NULL;
-	foreach_ptr(shardInterval, shardIntervalList)
+	foreach_declared_ptr(shardInterval, shardIntervalList)
 	{
 		/*
 		 * For each shard we first create the shard table in a separate
@@ -1475,7 +1486,7 @@ CopyShardTablesViaBlockWrites(List *shardIntervalList, char *sourceNodeName,
 		sourceNodePort,
 		PLACEMENT_UPDATE_STATUS_CREATING_CONSTRAINTS);
 
-	foreach_ptr(shardInterval, shardIntervalList)
+	foreach_declared_ptr(shardInterval, shardIntervalList)
 	{
 		List *ddlCommandList =
 			PostLoadShardCreationCommandList(shardInterval, sourceNodeName,
@@ -1492,7 +1503,7 @@ CopyShardTablesViaBlockWrites(List *shardIntervalList, char *sourceNodeName,
 	 * Create DDL commands to Attach child tables to their parents in a partitioning hierarchy.
 	 */
 	List *shardIntervalWithDDCommandsList = NIL;
-	foreach_ptr(shardInterval, shardIntervalList)
+	foreach_declared_ptr(shardInterval, shardIntervalList)
 	{
 		if (PartitionTable(shardInterval->relationId))
 		{
@@ -1517,7 +1528,7 @@ CopyShardTablesViaBlockWrites(List *shardIntervalList, char *sourceNodeName,
 	 * Iterate through the colocated shards and create DDL commamnds
 	 * to create the foreign constraints.
 	 */
-	foreach_ptr(shardInterval, shardIntervalList)
+	foreach_declared_ptr(shardInterval, shardIntervalList)
 	{
 		List *shardForeignConstraintCommandList = NIL;
 		List *referenceTableForeignConstraintList = NIL;
@@ -1536,7 +1547,7 @@ CopyShardTablesViaBlockWrites(List *shardIntervalList, char *sourceNodeName,
 
 	/* Now execute the Partitioning & Foreign constraints creation commads. */
 	ShardCommandList *shardCommandList = NULL;
-	foreach_ptr(shardCommandList, shardIntervalWithDDCommandsList)
+	foreach_declared_ptr(shardCommandList, shardIntervalWithDDCommandsList)
 	{
 		char *tableOwner = TableOwner(shardCommandList->shardInterval->relationId);
 		SendCommandListToWorkerOutsideTransaction(targetNodeName, targetNodePort,
@@ -1566,7 +1577,7 @@ CopyShardsToNode(WorkerNode *sourceNode, WorkerNode *targetNode, List *shardInte
 	int taskId = 0;
 	List *copyTaskList = NIL;
 	ShardInterval *shardInterval = NULL;
-	foreach_ptr(shardInterval, shardIntervalList)
+	foreach_declared_ptr(shardInterval, shardIntervalList)
 	{
 		/*
 		 * Skip copying data for partitioned tables, because they contain no
@@ -1699,7 +1710,7 @@ SearchShardPlacementInList(List *shardPlacementList, const char *nodeName,
 						   uint32 nodePort)
 {
 	ShardPlacement *shardPlacement = NULL;
-	foreach_ptr(shardPlacement, shardPlacementList)
+	foreach_declared_ptr(shardPlacement, shardPlacementList)
 	{
 		if (strncmp(nodeName, shardPlacement->nodeName, MAX_NODE_LENGTH) == 0 &&
 			nodePort == shardPlacement->nodePort)
@@ -1820,7 +1831,7 @@ CopyShardForeignConstraintCommandListGrouped(ShardInterval *shardInterval,
 	*referenceTableForeignConstraintList = NIL;
 
 	const char *command = NULL;
-	foreach_ptr(command, commandList)
+	foreach_declared_ptr(command, commandList)
 	{
 		char *escapedCommand = quote_literal_cstr(command);
 
@@ -1940,11 +1951,7 @@ ConstructQualifiedShardName(ShardInterval *shardInterval)
 static List *
 RecreateTableDDLCommandList(Oid relationId)
 {
-	const char *relationName = get_rel_name(relationId);
-	Oid relationSchemaId = get_rel_namespace(relationId);
-	const char *relationSchemaName = get_namespace_name(relationSchemaId);
-	const char *qualifiedRelationName = quote_qualified_identifier(relationSchemaName,
-																   relationName);
+	const char *qualifiedRelationName = generate_qualified_relation_name(relationId);
 
 	StringInfo dropCommand = makeStringInfo();
 
@@ -1989,7 +1996,7 @@ DropShardPlacementsFromMetadata(List *shardList,
 								char *nodeName, int32 nodePort)
 {
 	ShardInterval *shardInverval = NULL;
-	foreach_ptr(shardInverval, shardList)
+	foreach_declared_ptr(shardInverval, shardList)
 	{
 		uint64 shardId = shardInverval->shardId;
 		List *shardPlacementList = ShardPlacementList(shardId);
@@ -2053,7 +2060,7 @@ WorkerApplyShardDDLCommandList(List *ddlCommandList, int64 shardId)
 	List *applyDDLCommandList = NIL;
 
 	TableDDLCommand *ddlCommand = NULL;
-	foreach_ptr(ddlCommand, ddlCommandList)
+	foreach_declared_ptr(ddlCommand, ddlCommandList)
 	{
 		Assert(CitusIsA(ddlCommand, TableDDLCommand));
 		char *applyDDLCommand = GetShardedTableDDLCommand(ddlCommand, shardId, NULL);
@@ -2087,7 +2094,7 @@ UpdatePlacementUpdateStatusForShardIntervalList(List *shardIntervalList,
 	}
 
 	ProgressMonitorData *monitor = NULL;
-	foreach_ptr(monitor, rebalanceMonitorList)
+	foreach_declared_ptr(monitor, rebalanceMonitorList)
 	{
 		PlacementUpdateEventProgress *steps = ProgressMonitorSteps(monitor);
 
@@ -2098,7 +2105,7 @@ UpdatePlacementUpdateStatusForShardIntervalList(List *shardIntervalList,
 			bool foundInList = false;
 
 			ShardInterval *candidateShard = NULL;
-			foreach_ptr(candidateShard, shardIntervalList)
+			foreach_declared_ptr(candidateShard, shardIntervalList)
 			{
 				if (candidateShard->shardId == currentShardId)
 				{
