@@ -1170,7 +1170,8 @@ HasComplexRangeTableType(Query *queryTree)
 		if (rangeTableEntry->rtekind != RTE_RELATION &&
 			rangeTableEntry->rtekind != RTE_SUBQUERY &&
 			rangeTableEntry->rtekind != RTE_FUNCTION &&
-			rangeTableEntry->rtekind != RTE_VALUES)
+			rangeTableEntry->rtekind != RTE_VALUES &&
+			!IsJsonTableRTE(rangeTableEntry))
 		{
 			hasComplexRangeTableType = true;
 		}
@@ -1414,7 +1415,7 @@ IsJoinClause(Node *clause)
 	}
 	Var *initialVar = castNode(Var, linitial(varList));
 
-	foreach_ptr(var, varList)
+	foreach_declared_ptr(var, varList)
 	{
 		if (var->varno != initialVar->varno)
 		{
