@@ -6,9 +6,7 @@ citus_tables_create_query=$CTCQ$
     CREATE OR REPLACE VIEW %I.citus_tables AS
     SELECT
         logicalrelid AS table_name,
-        CASE WHEN partkey IS NOT NULL THEN 'distributed' ELSE
-            CASE when repmodel = 't' THEN 'reference' ELSE 'local' END
-        END AS citus_table_type,
+        CASE WHEN partkey IS NOT NULL THEN 'distributed' ELSE 'reference' END AS citus_table_type,
         coalesce(column_to_column_name(logicalrelid, partkey), '<none>') AS distribution_column,
         colocationid AS colocation_id,
         pg_size_pretty(citus_total_relation_size(logicalrelid, fail_on_error := false)) AS table_size,
@@ -22,15 +20,7 @@ citus_tables_create_query=$CTCQ$
     LEFT JOIN
         pg_am a ON (a.oid = c.relam)
     WHERE
-        -- filter out tables owned by extensions
-        logicalrelid NOT IN (
-            SELECT
-                objid
-            FROM
-                pg_depend
-            WHERE
-                classid = 'pg_class'::regclass AND refclassid = 'pg_extension'::regclass AND deptype = 'e'
-        )
+        partkey IS NOT NULL OR repmodel = 't'
     ORDER BY
         logicalrelid::text;
 $CTCQ$;
