@@ -10,7 +10,8 @@ CREATE OR REPLACE FUNCTION pg_catalog.get_rebalance_table_shards_plan(
         max_shard_moves int default 1000000,
         excluded_shard_list bigint[] default '{}',
         drain_only boolean default false,
-        rebalance_strategy name default NULL
+        rebalance_strategy name default NULL,
+        improvement_threshold float4 DEFAULT NULL
     )
     RETURNS TABLE (table_name regclass,
                    shardid bigint,
@@ -21,6 +22,6 @@ CREATE OR REPLACE FUNCTION pg_catalog.get_rebalance_table_shards_plan(
                    targetport int)
     AS 'MODULE_PATHNAME'
     LANGUAGE C VOLATILE;
-COMMENT ON FUNCTION pg_catalog.get_rebalance_table_shards_plan(regclass, float4, int, bigint[], boolean, name)
+COMMENT ON FUNCTION pg_catalog.get_rebalance_table_shards_plan(regclass, float4, int, bigint[], boolean, name, float4)
     IS 'returns the list of shard placement moves to be done on a rebalance operation';
 
