@@ -72,7 +72,7 @@ SendCommandToWorkersAsUser(TargetWorkerSet targetWorkerSet, const char *nodeUser
 
 	/* run commands serially */
 	WorkerNode *workerNode = NULL;
-	foreach_ptr(workerNode, workerNodeList)
+	foreach_declared_ptr(workerNode, workerNodeList)
 	{
 		const char *nodeName = workerNode->workerName;
 		int nodePort = workerNode->workerPort;
@@ -145,7 +145,7 @@ void
 SendCommandListToWorkersWithMetadata(List *commands)
 {
 	char *command = NULL;
-	foreach_ptr(command, commands)
+	foreach_declared_ptr(command, commands)
 	{
 		SendCommandToWorkersWithMetadata(command);
 	}
@@ -171,7 +171,7 @@ TargetWorkerSetNodeList(TargetWorkerSet targetWorkerSet, LOCKMODE lockMode)
 	List *result = NIL;
 
 	WorkerNode *workerNode = NULL;
-	foreach_ptr(workerNode, workerNodeList)
+	foreach_declared_ptr(workerNode, workerNodeList)
 	{
 		if ((targetWorkerSet == NON_COORDINATOR_METADATA_NODES || targetWorkerSet ==
 			 METADATA_NODES) &&
@@ -205,7 +205,7 @@ SendBareCommandListToMetadataWorkers(List *commandList)
 
 	/* run commands serially */
 	WorkerNode *workerNode = NULL;
-	foreach_ptr(workerNode, workerNodeList)
+	foreach_declared_ptr(workerNode, workerNodeList)
 	{
 		const char *nodeName = workerNode->workerName;
 		int nodePort = workerNode->workerPort;
@@ -218,7 +218,7 @@ SendBareCommandListToMetadataWorkers(List *commandList)
 
 		/* iterate over the commands and execute them in the same connection */
 		const char *commandString = NULL;
-		foreach_ptr(commandString, commandList)
+		foreach_declared_ptr(commandString, commandList)
 		{
 			ExecuteCriticalRemoteCommand(workerConnection, commandString);
 		}
@@ -271,7 +271,7 @@ SendCommandToWorkersParamsInternal(TargetWorkerSet targetWorkerSet, const char *
 
 	/* open connections in parallel */
 	WorkerNode *workerNode = NULL;
-	foreach_ptr(workerNode, workerNodeList)
+	foreach_declared_ptr(workerNode, workerNodeList)
 	{
 		const char *nodeName = workerNode->workerName;
 		int nodePort = workerNode->workerPort;
@@ -299,7 +299,7 @@ SendCommandToWorkersParamsInternal(TargetWorkerSet targetWorkerSet, const char *
 
 	/* send commands in parallel */
 	MultiConnection *connection = NULL;
-	foreach_ptr(connection, connectionList)
+	foreach_declared_ptr(connection, connectionList)
 	{
 		int querySent = SendRemoteCommandParams(connection, command, parameterCount,
 												parameterTypes, parameterValues, false);
@@ -310,7 +310,7 @@ SendCommandToWorkersParamsInternal(TargetWorkerSet targetWorkerSet, const char *
 	}
 
 	/* get results */
-	foreach_ptr(connection, connectionList)
+	foreach_declared_ptr(connection, connectionList)
 	{
 		PGresult *result = GetRemoteCommandResult(connection, true);
 		if (!IsResponseOK(result))
@@ -381,7 +381,7 @@ SendCommandListToWorkerOutsideTransactionWithConnection(MultiConnection *workerC
 
 	/* iterate over the commands and execute them in the same connection */
 	const char *commandString = NULL;
-	foreach_ptr(commandString, commandList)
+	foreach_declared_ptr(commandString, commandList)
 	{
 		ExecuteCriticalRemoteCommand(workerConnection, commandString);
 	}
@@ -422,7 +422,7 @@ SendCommandListToWorkerListWithBareConnections(List *workerConnectionList,
 
 	/* send commands in parallel */
 	MultiConnection *connection = NULL;
-	foreach_ptr(connection, workerConnectionList)
+	foreach_declared_ptr(connection, workerConnectionList)
 	{
 		int querySent = SendRemoteCommand(connection, stringToSend);
 		if (querySent == 0)
@@ -432,7 +432,7 @@ SendCommandListToWorkerListWithBareConnections(List *workerConnectionList,
 	}
 
 	bool failOnError = true;
-	foreach_ptr(connection, workerConnectionList)
+	foreach_declared_ptr(connection, workerConnectionList)
 	{
 		ClearResults(connection, failOnError);
 	}
@@ -462,7 +462,7 @@ SendMetadataCommandListToWorkerListInCoordinatedTransaction(List *workerNodeList
 	List *connectionList = NIL;
 
 	WorkerNode *workerNode = NULL;
-	foreach_ptr(workerNode, workerNodeList)
+	foreach_declared_ptr(workerNode, workerNodeList)
 	{
 		const char *nodeName = workerNode->workerName;
 		int nodePort = workerNode->workerPort;
@@ -499,7 +499,7 @@ SendMetadataCommandListToWorkerListInCoordinatedTransaction(List *workerNodeList
 	/* send commands in parallel */
 	bool failOnError = true;
 	MultiConnection *connection = NULL;
-	foreach_ptr(connection, connectionList)
+	foreach_declared_ptr(connection, connectionList)
 	{
 		int querySent = SendRemoteCommand(connection, stringToSend);
 		if (querySent == 0)
@@ -508,7 +508,7 @@ SendMetadataCommandListToWorkerListInCoordinatedTransaction(List *workerNodeList
 		}
 	}
 
-	foreach_ptr(connection, connectionList)
+	foreach_declared_ptr(connection, connectionList)
 	{
 		ClearResults(connection, failOnError);
 	}
@@ -537,7 +537,7 @@ SendOptionalCommandListToWorkerOutsideTransactionWithConnection(
 	/* iterate over the commands and execute them in the same connection */
 	bool failed = false;
 	const char *commandString = NULL;
-	foreach_ptr(commandString, commandList)
+	foreach_declared_ptr(commandString, commandList)
 	{
 		if (ExecuteOptionalRemoteCommand(workerConnection, commandString, NULL) != 0)
 		{
@@ -613,7 +613,7 @@ SendOptionalMetadataCommandListToWorkerInCoordinatedTransaction(const char *node
 
 	/* iterate over the commands and execute them in the same connection */
 	const char *commandString = NULL;
-	foreach_ptr(commandString, commandList)
+	foreach_declared_ptr(commandString, commandList)
 	{
 		if (ExecuteOptionalRemoteCommand(workerConnection, commandString, NULL) !=
 			RESPONSE_OKAY)
@@ -648,7 +648,7 @@ static void
 ErrorIfAnyMetadataNodeOutOfSync(List *metadataNodeList)
 {
 	WorkerNode *metadataNode = NULL;
-	foreach_ptr(metadataNode, metadataNodeList)
+	foreach_declared_ptr(metadataNode, metadataNodeList)
 	{
 		Assert(metadataNode->hasMetadata);
 
