@@ -1,11 +1,3 @@
-SHOW server_version \gset
-SELECT substring(:'server_version', '\d+')::int >= 15 AS server_version_ge_15
-\gset
-\if :server_version_ge_15
-\else
-\q
-\endif
-
 -- MERGE command performs a join from data_source to target_table_name
 DROP SCHEMA IF EXISTS merge_schema CASCADE;
 --MERGE INTO target
