@@ -3,12 +3,6 @@
 --
 
 -- Tests to check if we inform the user about potential caveats of creating new
--- databases, schemas, and roles.
+-- databases.
 
 CREATE DATABASE new_database;
-
-CREATE SCHEMA new_schema;
-
-CREATE ROLE new_role;
-
-CREATE USER new_user;
