@@ -25,7 +25,7 @@ WITH event_id
          FROM   events_table)
 SELECT Count(*)
 FROM   event_id
-WHERE  events_user_id IN (SELECT user_id
+WHERE  (events_user_id, random()) IN (SELECT user_id, 1
                           FROM   users_table
                           WHERE  users_table.time = events_time);
 
@@ -847,8 +847,124 @@ WHERE (SELECT COUNT(DISTINCT e1.value_2)
      WHERE e1.user_id = u1.user_id
           ) > 115 AND false;
 
+-- Test redundant WHERE clause (fix #7782, #7783)
+CREATE TABLE t0 (vkey int4, pkey int4, c0 timestamp);
+CREATE TABLE t1 (vkey int4, pkey int4, c4 timestamp, c5 text, c6 text);
+CREATE TABLE t3 (vkey int4, pkey int4, c9 timestamp);
+CREATE TABLE t7 (vkey int4, pkey int4);
+
+-- DEBUG messages not needed for these tests
 SET client_min_messages TO DEFAULT;
 
+INSERT INTO t0 (vkey, pkey, c0) values
+(3, 13000, make_timestamp(2032, 9, 4, 13, 38, 0));
+
+INSERT INTO t7 (vkey, pkey) values
+(3, 59525);
+
+SELECT create_reference_table('t1');
+SELECT create_distributed_table('t3', 'c9');
+
+UPDATE t0 set vkey = 117
+where (((t0.pkey) in (select t7.vkey from t7 where false
+        union all
+        select t3.pkey from t3 where false
+        )))
+  or TRUE;
+
+-- Local table t0 is updated
+SELECT vkey, pkey, c0 FROM t0;
+
+-- MERGE command with redundant join can be planned locally
+EXPLAIN (costs off, timing off)
+MERGE INTO t0 USING t7 ON
+ (((t0.pkey) in (select t7.vkey from t7 where false
+        union all
+        select t1.pkey from t1 where false
+        )))
+  or TRUE
+WHEN MATCHED THEN
+   UPDATE SET vkey = 113;
+
+-- UPDATE via MERGE with redundant join clause:
+MERGE INTO t0 USING t7 ON
+ (((t0.pkey) in (select t7.vkey from t7 where false
+        union all
+        select t1.pkey from t1 where false
+        )))
+  or TRUE
+WHEN MATCHED THEN
+   UPDATE SET vkey = 113;
+
+-- Local table t0 is updated
+SELECT vkey, pkey, c0 FROM t0;
+
+DELETE FROM t0
+where TRUE or (((t0.vkey) >= (select
+          pg_catalog.regexp_count(ref_0.c5, ref_0.c6)
+        from t1 as ref_0 where true)));
+
+-- Local table t0 is now empty (0 rows)
+SELECT vkey, pkey, c0 FROM t0;
+
+INSERT INTO t3 (vkey, pkey, c9) values
+(3, 13000, make_timestamp(2032, 9, 4, 13, 38, 0));
+
+-- Distributed table update with redundant WHERE
+UPDATE t3 set vkey = 117
+where (((t3.pkey) in (select t1.vkey from t1 where false
+        union all
+        select t0.pkey from t0 join t7 on t0.pkey=t7.vkey where false
+        )))
+  or TRUE;
+
+SELECT vkey, pkey FROM t3;
+
+-- Distributed table delete with redundant WHERE
+DELETE FROM t3
+where TRUE or (((t3.vkey) >= (select
+          pg_catalog.regexp_count(ref_0.c5, ref_0.c6)
+        from  t1 as ref_0  where true)) and (select max(vkey) from t0) > 0);
+
+-- Distributed table t3 is now empty
+SELECT vkey, pkey FROM t3;
+
+-- Redundant WHERE clause with distributed parititioned table
+CREATE TABLE a (a int);
+INSERT INTO a VALUES (1);
+
+-- populated distributed partitioned table
+create table partitioned_table (a INT UNIQUE) PARTITION BY RANGE(a);
+CREATE TABLE par_1 PARTITION OF partitioned_table FOR VALUES FROM (1) TO (41);
+CREATE TABLE par_2 PARTITION OF partitioned_table FOR VALUES FROM (41) TO (81);
+CREATE TABLE par_3 PARTITION OF partitioned_table FOR VALUES FROM (81) TO (121);
+CREATE TABLE par_4 PARTITION OF partitioned_table FOR VALUES FROM (121) TO (161);
+SELECT create_distributed_table('partitioned_table', 'a');
+insert into partitioned_table(a) select  i from generate_series(1,160) i;
+
+-- test citus table in init plan
+-- with redundant WHERE clause
+SELECT CASE WHEN EXISTS (
+       SELECT * FROM partitioned_table
+   ) THEN 1 ELSE 0 END AS table_non_empty
+FROM   a
+WHERE true;
+
+-- test citus table in init plan
+-- with redundant WHERE clause involving
+-- a citus table
+SELECT CASE WHEN EXISTS (
+       SELECT * FROM partitioned_table
+   ) THEN 1 ELSE 0 END AS table_non_empty
+FROM   a
+WHERE true OR NOT EXISTS (SELECT 1 FROM t1);
+
 DROP TABLE local_table;
+DROP TABLE t0;
+DROP TABLE t1;
+DROP TABLE t3;
+DROP TABLE t7;
+DROP TABLE a;
+DROP TABLE partitioned_table CASCADE;
 DROP SCHEMA subquery_in_where CASCADE;
 SET search_path TO public;
